package com.ruoyi.system.service.impl;

import cn.dev33.satoken.stp.StpUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.github.yulichang.base.MPJBaseServiceImpl;
import com.github.yulichang.wrapper.MPJLambdaWrapper;
import com.ruoyi.common.annotation.DataScope;
import com.ruoyi.common.constant.UserConstants;
import com.ruoyi.common.core.domain.TreeSelect;
import com.ruoyi.common.core.domain.entity.SysDept;
import com.ruoyi.common.core.domain.entity.SysRole;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.core.text.Convert;
import com.ruoyi.common.enums.UserStatus;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.spring.SpringUtils;
import com.ruoyi.system.domain.SysRoleDept;
import com.ruoyi.system.mapper.SysDeptMapper;
import com.ruoyi.system.mapper.SysRoleMapper;
import com.ruoyi.system.mapper.SysUserMapper;
import com.ruoyi.system.service.ISysDeptService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 部门管理 服务实现
 * 
 * @author ruoyi
 */
@Service
public class SysDeptServiceImpl extends MPJBaseServiceImpl<SysDeptMapper, SysDept> implements ISysDeptService
{
    @Resource
    private SysRoleMapper roleMapper;

    @Resource
    private SysUserMapper userMapper;

    /**
     * 查询部门管理数据
     * 
     * @param dept 部门信息
     * @return 部门信息集合
     */
    @Override
    @DataScope(deptAlias = "d")
    public List<SysDept> selectDeptList(SysDept dept)
    {
        MPJLambdaWrapper<SysDept> sysDeptLambdaQueryWrapper = new MPJLambdaWrapper<SysDept>("d")
                .selectAll(SysDept.class)
                .eq(StringUtils.isNotLongNull(dept.getDeptId()),SysDept::getDeptId,dept.getDeptId())
                .eq(StringUtils.isNotLongNull(dept.getParentId()),SysDept::getParentId,dept.getParentId())
                .like(StringUtils.isNotBlank(dept.getDeptName()),SysDept::getDeptName,dept.getDeptName())
                .eq(StringUtils.isNotBlank(dept.getStatus()),SysDept::getStatus,dept.getStatus())
                .apply(StringUtils.isNotBlank((String)dept.getParams().get("dataScope")), (String) dept.getParams().get("dataScope"));
        return selectJoinList(SysDept.class, sysDeptLambdaQueryWrapper);
    }

    /**
     * 查询部门树结构信息
     * 
     * @param dept 部门信息
     * @return 部门树信息集合
     */
    @Override
    public List<TreeSelect> selectDeptTreeList(SysDept dept)
    {
        List<SysDept> depts = SpringUtils.getAopProxy(this).selectDeptList(dept);
        return buildDeptTreeSelect(depts);
    }

    /**
     * 构建前端所需要树结构
     * 
     * @param depts 部门列表
     * @return 树结构列表
     */
    @Override
    public List<SysDept> buildDeptTree(List<SysDept> depts)
    {
        List<SysDept> returnList = new ArrayList<SysDept>();
        List<Long> tempList = depts.stream().map(SysDept::getDeptId).collect(Collectors.toList());
        for (SysDept dept : depts)
        {
            // 如果是顶级节点, 遍历该父节点的所有子节点
            if (!tempList.contains(dept.getParentId()))
            {
                recursionFn(depts, dept);
                returnList.add(dept);
            }
        }
        if (returnList.isEmpty())
        {
            returnList = depts;
        }
        return returnList;
    }

    /**
     * 构建前端所需要下拉树结构
     * 
     * @param depts 部门列表
     * @return 下拉树结构列表
     */
    @Override
    public List<TreeSelect> buildDeptTreeSelect(List<SysDept> depts)
    {
        List<SysDept> deptTrees = buildDeptTree(depts);
        return deptTrees.stream().map(TreeSelect::new).collect(Collectors.toList());
    }

    /**
     * 根据角色ID查询部门树信息
     * 
     * @param roleId 角色ID
     * @return 选中部门列表
     */
    @Override
    public List<Long> selectDeptListByRoleId(Long roleId)
    {
        SysRole role = roleMapper.selectById(roleId);
        List<Long> parentId = new ArrayList<>();
        if (role.isDeptCheckStrictly()){
            MPJLambdaWrapper<SysDept> sysDeptMPJLambdaWrapper = new MPJLambdaWrapper<SysDept>("d")
                    .select(SysDept::getParentId)
                    .innerJoin(SysRoleDept.class,SysRoleDept::getDeptId,SysDept::getDeptId)
                    .eq(SysRoleDept::getRoleId,roleId);
            List<SysDept> sysDepts = selectJoinList(SysDept.class, sysDeptMPJLambdaWrapper);
            parentId = sysDepts.stream().map(SysDept::getParentId).collect(Collectors.toList());

        }
        MPJLambdaWrapper<SysDept> sysDeptMPJLambdaQueryWrapper = new MPJLambdaWrapper<SysDept>("d")
                .select(SysDept::getDeptId)
                .leftJoin(SysRoleDept.class,"rd",SysRoleDept::getDeptId,SysDept::getDeptId)
                .notIn(role.isDeptCheckStrictly() && parentId.size() > 0,SysDept::getDeptId,parentId)
                .orderByAsc(SysDept::getParentId)
                .orderByAsc(SysDept::getOrderNum);
        List<SysDept> sysDepts = selectJoinList(SysDept.class, sysDeptMPJLambdaQueryWrapper);
        return sysDepts.stream().map(SysDept::getDeptId).collect(Collectors.toList());
    }

    /**
     * 根据部门ID查询信息
     * 
     * @param deptId 部门ID
     * @return 部门信息
     */
    @Override
    public SysDept selectDeptById(Long deptId)
    {
        MPJLambdaWrapper<SysDept> sysDeptMPJLambdaQueryWrapper = new MPJLambdaWrapper<SysDept>("d")
                .selectAll(SysDept.class)
                .eq(SysDept::getDeptId,deptId)
                .selectSub(SysDept.class,w -> w
                        .select(SysDept::getDeptName)
                        .eq(SysDept::getDeptId,SysDept::getParentId)
                        .last("limit 1"),SysDept::getParentName)
                .last("limit 1");
        return selectJoinOne(SysDept.class, sysDeptMPJLambdaQueryWrapper);
    }

    /**
     * 根据ID查询所有子部门（正常状态）
     * 
     * @param deptId 部门ID
     * @return 子部门数
     */
    @Override
    public long selectNormalChildrenDeptById(Long deptId)
    {
        LambdaQueryWrapper<SysDept> sysDeptLambdaQueryWrapper = new LambdaQueryWrapper<SysDept>()
                .eq(SysDept::getStatus,"0")
                .apply("find_in_set("+deptId+", ancestors)");

        return count(sysDeptLambdaQueryWrapper);
    }

    /**
     * 是否存在子节点
     * 
     * @param deptId 部门ID
     * @return 结果
     */
    @Override
    public boolean hasChildByDeptId(Long deptId)
    {
        LambdaQueryWrapper<SysDept> hasChild = new LambdaQueryWrapper<SysDept>()
                .eq(SysDept::getParentId, deptId)
                .last("limit 1");
        long result = count(hasChild);
        return result > 0;
    }

    /**
     * 查询部门是否存在用户
     * 
     * @param deptId 部门ID
     * @return 结果 true 存在 false 不存在
     */
    @Override
    public boolean checkDeptExistUser(Long deptId)
    {
        LambdaQueryWrapper<SysUser> hasChild = new LambdaQueryWrapper<SysUser>()
                .eq(SysUser::getDeptId, deptId);
        long result = userMapper.selectCount(hasChild);
        return result > 0;
    }

    /**
     * 校验部门名称是否唯一
     * 
     * @param dept 部门信息
     * @return 结果
     */
    @Override
    public boolean checkDeptNameUnique(SysDept dept)
    {
        Long deptId = StringUtils.isNull(dept.getDeptId()) ? -1L : dept.getDeptId();
        LambdaQueryWrapper<SysDept> checkDeptName = new LambdaQueryWrapper<SysDept>()
                .eq(SysDept::getDeptName,dept.getDeptName())
                .eq(SysDept::getParentId, dept.getParentId())
                .last("limit 1");
        SysDept info = getOne(checkDeptName);
        if (StringUtils.isNotNull(info) && info.getDeptId().longValue() != deptId.longValue())
        {
            return UserConstants.NOT_UNIQUE;
        }
        return UserConstants.UNIQUE;
    }

    /**
     * 校验部门是否有数据权限
     * 
     * @param deptId 部门id
     */
    @Override
    public void checkDeptDataScope(Long deptId)
    {
        if (!SysUser.isAdmin(Long.parseLong((String) StpUtil.getLoginId())))
        {
            SysDept dept = new SysDept();
            dept.setDeptId(deptId);
            List<SysDept> depts = SpringUtils.getAopProxy(this).selectDeptList(dept);
            if (StringUtils.isEmpty(depts))
            {
                throw new ServiceException("没有权限访问部门数据！");
            }
        }
    }

    /**
     * 新增保存部门信息
     * 
     * @param dept 部门信息
     * @return 结果
     */
    @Override
    public boolean insertDept(SysDept dept)
    {
        SysDept info = SpringUtils.getAopProxy(this).selectDeptById(dept.getParentId());
        // 如果父节点不为正常状态,则不允许新增子节点
        if (!UserConstants.DEPT_NORMAL.equals(info.getStatus()))
        {
            throw new ServiceException("部门停用，不允许新增");
        }
        dept.setAncestors(info.getAncestors() + "," + dept.getParentId());
        return save(dept);
    }

    /**
     * 修改保存部门信息
     * 
     * @param dept 部门信息
     * @return 结果
     */
    @Override
    public boolean updateDept(SysDept dept)
    {
        SysDept newParentDept = SpringUtils.getAopProxy(this).selectDeptById(dept.getParentId());
        SysDept oldDept = SpringUtils.getAopProxy(this).selectDeptById(dept.getDeptId());
        if (StringUtils.isNotNull(newParentDept) && StringUtils.isNotNull(oldDept))
        {
            String newAncestors = newParentDept.getAncestors() + "," + newParentDept.getDeptId();
            String oldAncestors = oldDept.getAncestors();
            dept.setAncestors(newAncestors);
            updateDeptChildren(dept.getDeptId(), newAncestors, oldAncestors);
        }
        boolean result = updateById(dept);
        if (UserConstants.DEPT_NORMAL.equals(dept.getStatus()) && StringUtils.isNotEmpty(dept.getAncestors())
                && !StringUtils.equals("0", dept.getAncestors()))
        {
            // 如果该部门是启用状态，则启用该部门的所有上级部门
            updateParentDeptStatusNormal(dept);
        }
        return result;
    }

    /**
     * 修改该部门的父级部门状态
     * 
     * @param dept 当前部门
     */
    private void updateParentDeptStatusNormal(SysDept dept)
    {
        String ancestors = dept.getAncestors();
        Long[] deptIds = Convert.toLongArray(ancestors);
        LambdaUpdateWrapper<SysDept> updateParentDeptStatus = new LambdaUpdateWrapper<SysDept>()
                .set(SysDept::getStatus, UserStatus.OK.getCode())
                .in(SysDept::getDeptId, Arrays.asList(deptIds));
        update(updateParentDeptStatus);
    }

    /**
     * 修改子元素关系
     * 
     * @param deptId 被修改的部门ID
     * @param newAncestors 新的父ID集合
     * @param oldAncestors 旧的父ID集合
     */
    public void updateDeptChildren(Long deptId, String newAncestors, String oldAncestors)
    {
        List<SysDept> children = list(new LambdaQueryWrapper<SysDept>()
                .apply("find_in_set("+deptId+", ancestors)"));
        if (children == null || children.size() == 0){
            return;
        }
        for (SysDept child : children)
        {
            LambdaUpdateWrapper<SysDept> updateChild = new LambdaUpdateWrapper<SysDept>()
                    .set(SysDept::getAncestors, child.getAncestors().replaceFirst(oldAncestors, newAncestors))
                    .eq(SysDept::getDeptId, child.getDeptId());
            update(updateChild);
        }

    }

    /**
     * 删除部门管理信息
     * 
     * @param deptId 部门ID
     * @return 结果
     */
    @Override
    public boolean deleteDeptById(Long deptId)
    {
        return removeById(deptId);
    }

    /**
     * 递归列表
     */
    private void recursionFn(List<SysDept> list, SysDept t)
    {
        // 得到子节点列表
        List<SysDept> childList = getChildList(list, t);
        t.setChildren(childList);
        for (SysDept tChild : childList)
        {
            if (hasChild(list, tChild))
            {
                recursionFn(list, tChild);
            }
        }
    }

    /**
     * 得到子节点列表
     */
    private List<SysDept> getChildList(List<SysDept> list, SysDept t)
    {
        List<SysDept> tlist = new ArrayList<SysDept>();
        Iterator<SysDept> it = list.iterator();
        while (it.hasNext())
        {
            SysDept n = (SysDept) it.next();
            if (StringUtils.isNotNull(n.getParentId()) && n.getParentId().longValue() == t.getDeptId().longValue())
            {
                tlist.add(n);
            }
        }
        return tlist;
    }

    /**
     * 判断是否有子节点
     */
    private boolean hasChild(List<SysDept> list, SysDept t)
    {
        return getChildList(list, t).size() > 0;
    }
}
